@@ -21,11 +21,22 @@
 #include <mousepad/mousepad-prefs-dialog.h>
 #include <mousepad/mousepad-replace-dialog.h>
 #include <mousepad/mousepad-window.h>
+#include <mousepad/mousepad-util.h>
 
 #include <xfconf/xfconf.h>
 
 
 
+/* GObject virtual functions */
+static void        mousepad_application_set_property              (GObject                  *object,
+                                                                   guint                     prop_id,
+                                                                   const GValue             *value,
+                                                                   GParamSpec               *pspec);
+static void        mousepad_application_get_property              (GObject                  *object,
+                                                                   guint                     prop_id,
+                                                                   GValue                   *value,
+                                                                   GParamSpec               *pspec);
+
 /* GApplication virtual functions, in the order in which they are called on launch */
 static gint        mousepad_application_handle_local_options      (GApplication             *gapplication,
                                                                    GVariantDict             *options);
@@ -49,11 +60,25 @@ static void        mousepad_application_new_window_with_document  (MousepadWindo
 static void        mousepad_application_new_window                (MousepadWindow           *existing,
                                                                    MousepadApplication      *application);
 static void        mousepad_application_active_window_changed     (MousepadApplication      *application);
+static void        mousepad_application_set_shared_menu_parts     (MousepadApplication      *application,
+                                                                   GMenuModel               *model);
 static void        mousepad_application_create_languages_menu     (MousepadApplication      *application);
 static void        mousepad_application_create_style_schemes_menu (MousepadApplication      *application);
 static void        mousepad_application_action_quit               (GSimpleAction            *action,
                                                                    GVariant                 *value,
                                                                    gpointer                  data);
+static void        mousepad_application_toggle_activate           (GSimpleAction            *action,
+                                                                   GVariant                 *parameter,
+                                                                   gpointer                  data);
+static void        mousepad_application_radio_activate            (GSimpleAction            *action,
+                                                                   GVariant                 *parameter,
+                                                                   gpointer                  data);
+static void        mousepad_application_action_update             (MousepadApplication      *application,
+                                                                   gchar                    *key,
+                                                                   GSettings                *settings);
+static void        mousepad_application_action_whitespace         (GSimpleAction            *action,
+                                                                   GVariant                 *state,
+                                                                   gpointer                  data);
 
 
 
@@ -66,11 +91,19 @@ struct _MousepadApplication
 {
   GtkApplication      __parent__;
 
-  /* the preferences dialog when shown */
-  GtkWidget *prefs_dialog;
+  /* preferences dialog related */
+  GtkWidget                   *prefs_dialog;
+  GtkSourceSpaceLocationFlags  space_location_flags;
 
   /* opening mode provided on the command line */
-  gint       opening_mode;
+  gint                         opening_mode;
+};
+
+/* MousepadApplication properties */
+enum
+{
+  PROP_SPACE_LOCATION=1,
+  N_PROPERTIES
 };
 
 
@@ -101,16 +134,6 @@ static const GOptionEntry option_entries[] =
   { NULL }
 };
 
-
-
-/* application actions */
-static const GActionEntry action_entries[] =
-{
-  { "quit", mousepad_application_action_quit, NULL, NULL, NULL }
-};
-
-
-
 /* opening mode provided on the command line */
 enum
 {
@@ -121,6 +144,82 @@ enum
 
 
 
+/* application actions */
+
+/* stateless actions */
+static const GActionEntry stateless_actions[] =
+{
+  /* command line options */
+  { "quit", mousepad_application_action_quit, NULL, NULL, NULL }
+};
+#define N_STATELESS G_N_ELEMENTS (stateless_actions)
+
+/* preferences dialog */
+static const GActionEntry dialog_actions[] =
+{
+  /* "View" tab */
+  { MOUSEPAD_SETTING_SHOW_LINE_NUMBERS, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_SHOW_WHITESPACE, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_SHOW_LINE_ENDINGS, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_SHOW_RIGHT_MARGIN, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_HIGHLIGHT_CURRENT_LINE, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_MATCH_BRACES, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_WORD_WRAP, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_USE_DEFAULT_FONT, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_COLOR_SCHEME, mousepad_application_radio_activate, "s", "'none'", NULL },
+
+  /* "Editor" tab */
+  { MOUSEPAD_SETTING_INSERT_SPACES, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_AUTO_INDENT, mousepad_application_toggle_activate, NULL, "false", NULL },
+
+  /* "Window" tab */
+  { MOUSEPAD_SETTING_STATUSBAR_VISIBLE, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_PATH_IN_TITLE, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_REMEMBER_SIZE, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_REMEMBER_POSITION, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_REMEMBER_STATE, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_TOOLBAR_VISIBLE, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_ALWAYS_SHOW_TABS, mousepad_application_toggle_activate, NULL, "false", NULL },
+  { MOUSEPAD_SETTING_CYCLE_TABS, mousepad_application_toggle_activate, NULL, "false", NULL }
+};
+#define N_DIALOG G_N_ELEMENTS (dialog_actions)
+
+/* settings only accessible from the menubar */
+static const GActionEntry menubar_actions[] =
+{
+  { MOUSEPAD_SETTING_SEARCH_HIGHLIGHT_ALL, mousepad_application_toggle_activate, NULL, "false", NULL }
+};
+#define N_MENUBAR G_N_ELEMENTS (menubar_actions)
+
+/* whitespace location settings, only accessible from GSettings */
+static const GActionEntry whitespace_actions[] =
+{
+  { MOUSEPAD_SETTING_SHOW_WHITESPACE_LEADING, mousepad_application_toggle_activate, NULL,
+    "false", mousepad_application_action_whitespace },
+  { MOUSEPAD_SETTING_SHOW_WHITESPACE_INSIDE, mousepad_application_toggle_activate, NULL,
+    "false", mousepad_application_action_whitespace },
+  { MOUSEPAD_SETTING_SHOW_WHITESPACE_TRAILING, mousepad_application_toggle_activate, NULL,
+    "false", mousepad_application_action_whitespace },
+};
+#define N_WHITESPACE G_N_ELEMENTS (whitespace_actions)
+
+/* concatenate all setting actions */
+static const GActionEntry* setting_actions[] =
+{
+  dialog_actions,
+  menubar_actions,
+  whitespace_actions
+};
+static const guint n_setting_actions[] =
+{
+  N_DIALOG,
+  N_MENUBAR,
+  N_WHITESPACE
+};
+#define N_SETTING G_N_ELEMENTS (n_setting_actions)
+
+
+
 G_DEFINE_TYPE (MousepadApplication, mousepad_application, GTK_TYPE_APPLICATION)
 
 
@@ -128,14 +227,65 @@ G_DEFINE_TYPE (MousepadApplication, mousepad_application, GTK_TYPE_APPLICATION)
 static void
 mousepad_application_class_init (MousepadApplicationClass *klass)
 {
+  GObjectClass      *gobject_class = G_OBJECT_CLASS (klass);
   GApplicationClass *application_class = G_APPLICATION_CLASS (klass);
 
+  gobject_class->set_property = mousepad_application_set_property;
+  gobject_class->get_property = mousepad_application_get_property;
+
   application_class->handle_local_options = mousepad_application_handle_local_options;
   application_class->startup = mousepad_application_startup;
   application_class->command_line = mousepad_application_command_line;
   application_class->activate = mousepad_application_activate;
   application_class->open = mousepad_application_open;
   application_class->shutdown = mousepad_application_shutdown;
+
+  g_object_class_install_property (gobject_class, PROP_SPACE_LOCATION,
+    g_param_spec_flags ("space-location", "SpaceLocation", "The space location setting",
+                        GTK_SOURCE_TYPE_SPACE_LOCATION_FLAGS, GTK_SOURCE_SPACE_LOCATION_ALL,
+                        G_PARAM_READWRITE));
+}
+
+
+
+static void
+mousepad_application_set_property (GObject      *object,
+                                   guint         prop_id,
+                                   const GValue *value,
+                                   GParamSpec   *pspec)
+{
+  MousepadApplication *application = MOUSEPAD_APPLICATION (object);
+
+  switch (prop_id)
+    {
+    case PROP_SPACE_LOCATION:
+      application->space_location_flags = g_value_get_flags (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+
+
+static void
+mousepad_application_get_property (GObject    *object,
+                                   guint       prop_id,
+                                   GValue     *value,
+                                   GParamSpec *pspec)
+{
+  MousepadApplication *application = MOUSEPAD_APPLICATION (object);
+
+  switch (prop_id)
+    {
+    case PROP_SPACE_LOCATION:
+      g_value_set_flags (value, application->space_location_flags);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
 }
 
 
@@ -149,7 +299,7 @@ mousepad_application_init (MousepadApplication *application)
   g_set_application_name (_("Mousepad"));
 
   /* use the Mousepad icon as default for new windows */
-  gtk_window_set_default_icon_name ("org.xfce.mousepad");
+  gtk_window_set_default_icon_name (MOUSEPAD_ID);
 
   /* this option is added separately using g_strdup_printf() for the description, to be sure
    * that opening modes will be excluded from the translation (experience shows that using a
@@ -325,6 +475,9 @@ mousepad_application_set_accels (MousepadApplication *application)
   const gchar      *accels[] = { NULL, NULL };
   const gchar      *accel_maps[][2] =
   {
+    /* increase/decrease font size from keyboard/mouse */
+    { "win.increase-font-size", "<Control>plus" }, { "win.decrease-font-size", "<Control>minus" },
+
     /* "File" menu */
     { "win.file.new", "<Control>N" }, { "win.file.new-window", "<Control><Shift>N" },
     { "win.file.open", "<Control>O" }, { "win.file.save", "<Control>S" },
@@ -348,7 +501,7 @@ mousepad_application_set_accels (MousepadApplication *application)
     { "win.search.go-to", "<Control>L" },
 
     /* "View" menu */
-    { "win.view.menubar", "<Control>M" }, { "win.view.fullscreen", "F11" },
+    { "win.preferences.window.menubar-visible", "<Control>M" }, { "win.view.fullscreen", "F11" },
 
     /* "Document" menu */
     { "win.document.previous-tab", "<Control>Page_Up" }, { "win.document.next-tab", "<Control>Page_Down" },
@@ -404,20 +557,66 @@ static void
 mousepad_application_startup (GApplication *gapplication)
 {
   MousepadApplication *application = MOUSEPAD_APPLICATION (gapplication);
+  GVariant            *state;
+  GAction             *action;
+  GMenu               *menu;
+  guint                m, n;
 
   /* chain up to parent */
   G_APPLICATION_CLASS (mousepad_application_parent_class)->startup (gapplication);
 
   /* initialize application attributes */
   application->prefs_dialog = NULL;
+  application->space_location_flags = GTK_SOURCE_SPACE_LOCATION_ALL;
   application->opening_mode = TAB;
 
   /* initialize mousepad settings */
   mousepad_settings_init ();
 
   /* add application actions */
-  g_action_map_add_action_entries (G_ACTION_MAP (application), action_entries,
-                                   G_N_ELEMENTS (action_entries), application);
+  g_action_map_add_action_entries (G_ACTION_MAP (application), stateless_actions,
+                                   N_STATELESS, application);
+  for (m = 0; m < N_SETTING; m++)
+    g_action_map_add_action_entries (G_ACTION_MAP (application), setting_actions[m],
+                                     n_setting_actions[m], application);
+
+  /* associate flags to whitespace actions: to do after mapping and before initialization */
+  for (m = 0; m < N_WHITESPACE; m++)
+    {
+      action = g_action_map_lookup_action (G_ACTION_MAP (application), whitespace_actions[m].name);
+      mousepad_object_set_data (action, "flag", GUINT_TO_POINTER (1 << m));
+    }
+
+  for (m = 0; m < N_SETTING; m++)
+    for (n = 0; n < n_setting_actions[m]; n++)
+      {
+        /* sync the action state to its setting */
+        mousepad_setting_connect_object (setting_actions[m][n].name,
+                                         G_CALLBACK (mousepad_application_action_update),
+                                         application, G_CONNECT_SWAPPED);
+
+        /* initialize the action state */
+        state = mousepad_setting_get_variant (setting_actions[m][n].name);
+        g_action_group_change_action_state (G_ACTION_GROUP (application),
+                                            setting_actions[m][n].name, state);
+        g_variant_unref (state);
+      }
+
+  /* set shared menu parts in the application menus */
+  menu = gtk_application_get_menu_by_id (GTK_APPLICATION (application), "shared-sections");
+  mousepad_application_set_shared_menu_parts (application, G_MENU_MODEL (menu));
+
+  menu = gtk_application_get_menu_by_id (GTK_APPLICATION (application), "tab-menu");
+  mousepad_application_set_shared_menu_parts (application, G_MENU_MODEL (menu));
+
+  menu = gtk_application_get_menu_by_id (GTK_APPLICATION (application), "textview-menu");
+  mousepad_application_set_shared_menu_parts (application, G_MENU_MODEL (menu));
+
+  menu = gtk_application_get_menu_by_id (GTK_APPLICATION (application), "toolbar");
+  mousepad_application_set_shared_menu_parts (application, G_MENU_MODEL (menu));
+
+  menu = gtk_application_get_menu_by_id (GTK_APPLICATION (application), "menubar");
+  mousepad_application_set_shared_menu_parts (application, G_MENU_MODEL (menu));
 
   /* set accels for actions */
   mousepad_application_set_accels (application);
@@ -450,7 +649,7 @@ mousepad_application_command_line (GApplication            *gapplication,
   /* initialize xfconf */
   if (G_UNLIKELY (xfconf_init (&error) == FALSE))
     {
-      g_application_command_line_printerr (command_line, "%s\n", "Failed to initialize xfconf");
+      g_critical ("Failed to initialize xfconf: %s", error->message);
       g_error_free (error);
 
       return EXIT_FAILURE;
@@ -471,8 +670,9 @@ mousepad_application_command_line (GApplication            *gapplication,
       else
         {
           application->opening_mode = MOUSEPAD_SETTING_GET_ENUM (OPENING_MODE);
-          g_application_command_line_printerr (command_line, "%s\n",
-                                               "Invalid opening mode: ignored");
+          g_application_command_line_printerr (command_line,
+                                               "Invalid opening mode '%s': ignored\n",
+                                               opening_mode);
         }
     }
   /* use the opening mode stored in the settings */
@@ -568,7 +768,6 @@ mousepad_application_open (GApplication  *gapplication,
 {
   MousepadApplication *application = MOUSEPAD_APPLICATION (gapplication);
   GtkWidget           *window;
-  GError              *error = NULL;
   GPtrArray           *uris;
   gpointer            *data;
   gchar               *filename, *uri;
@@ -580,7 +779,7 @@ mousepad_application_open (GApplication  *gapplication,
   for (n = 0; n < n_files; n++)
     {
       uri = g_file_get_uri (files[n]);
-      filename = g_filename_from_uri (uri, NULL, &error);
+      filename = g_file_get_path (files[n]);
 
       /* there could be invalid uris when the call comes from gdbus */
       if (filename)
@@ -590,8 +789,7 @@ mousepad_application_open (GApplication  *gapplication,
         }
       else
         {
-          g_printerr ("%s\n", error->message);
-          g_clear_error (&error);
+          g_message ("Invalid URI: %s", uri);
           g_free (uri);
         }
 
@@ -709,9 +907,9 @@ mousepad_application_create_window (MousepadApplication *application)
   gtk_window_set_screen (GTK_WINDOW (window), gdk_screen_get_default ());
 
   /* connect signals */
-  g_signal_connect (G_OBJECT (window), "new-window-with-document",
+  g_signal_connect (window, "new-window-with-document",
                     G_CALLBACK (mousepad_application_new_window_with_document), application);
-  g_signal_connect (G_OBJECT (window), "new-window",
+  g_signal_connect (window, "new-window",
                     G_CALLBACK (mousepad_application_new_window), application);
 
   return window;
@@ -783,6 +981,9 @@ mousepad_application_active_window_changed (MousepadApplication *application)
     {
       /* update document dependent menu items */
       mousepad_window_update_document_menu_items (MOUSEPAD_WINDOW (app_windows->data));
+
+      /* update window dependent menu items */
+      mousepad_window_update_window_menu_items (MOUSEPAD_WINDOW (app_windows->data));
     }
 
   /* store a copy of the application windows list to compare at next call */
@@ -797,7 +998,7 @@ mousepad_application_prefs_dialog_response (MousepadApplication *application,
                                             gint                 response_id,
                                             MousepadPrefsDialog *dialog)
 {
-  gtk_widget_destroy (GTK_WIDGET (application->prefs_dialog));
+  gtk_widget_destroy (application->prefs_dialog);
   application->prefs_dialog = NULL;
 }
 
@@ -840,6 +1041,140 @@ mousepad_application_show_preferences (MousepadApplication *application,
 
 
 
+static void
+mousepad_application_update_menu (GMenuModel *shared_menu,
+                                  gint        position,
+                                  gint        removed,
+                                  gint        added,
+                                  GMenuModel *model)
+{
+  GMenuItem *item;
+  gint       n, index;
+
+  /* update the target menu when shared menu items are added or removed: some of them may
+   * be temporarily added/removed during an update process, of permanently added/removed */
+  for (n = 0; n < removed; n++)
+    {
+      index = position + n;
+      item = g_menu_item_new_from_model (shared_menu, index);
+      g_menu_remove (G_MENU (model), index);
+      g_object_unref (item);
+    }
+
+  for (n = 0; n < added; n++)
+    {
+      index = position + n;
+      item = g_menu_item_new_from_model (shared_menu, index);
+      g_menu_insert_item (G_MENU (model), index, item);
+      g_object_unref (item);
+    }
+}
+
+
+
+static void
+mousepad_application_update_menu_item (GMenuModel *shared_item,
+                                       gint        position,
+                                       gint        removed,
+                                       gint        added,
+                                       GMenuModel *model)
+{
+  GMenuItem *item;
+  GVariant  *value;
+  gint       n;
+
+  /* update the target menu item only when the shared menu item is added, that is at the end
+   * of its own update: an isolated shared menu item is not supposed to be permanently removed */
+  if (added)
+    {
+      value = g_menu_model_get_item_attribute_value (shared_item, 0, "item-share-id",
+                                                     G_VARIANT_TYPE_STRING);
+      n = GPOINTER_TO_INT (mousepad_object_get_data (model, g_variant_get_string (value, NULL)));
+      g_variant_unref (value);
+
+      item = g_menu_item_new_from_model (shared_item, 0);
+      g_menu_remove (G_MENU (model), n);
+      g_menu_insert_item (G_MENU (model), n, item);
+      g_object_unref (item);
+    }
+}
+
+
+
+static void
+mousepad_application_set_shared_menu_parts (MousepadApplication *application,
+                                            GMenuModel          *model)
+{
+  GMenuModel  *section, *shared_menu, *shared_item, *submodel;
+  GVariant    *value;
+  const gchar *share_id;
+  gint         n;
+
+  for (n = 0; n < g_menu_model_get_n_items (model); n++)
+    {
+      /* section GMenuItem with a share id: insert the shared menu */
+      if ((section = g_menu_model_get_item_link (model, n, G_MENU_LINK_SECTION))
+          && ((value = g_menu_model_get_item_attribute_value (model, n, "section-share-id",
+                                                              G_VARIANT_TYPE_STRING))))
+        {
+          share_id = g_variant_get_string (value, NULL);
+          g_variant_unref (value);
+
+          shared_menu = G_MENU_MODEL (gtk_application_get_menu_by_id (
+                                        GTK_APPLICATION (application), share_id));
+          mousepad_application_update_menu (shared_menu, 0, 0,
+                                            g_menu_model_get_n_items (shared_menu), section);
+
+          g_signal_connect_object (shared_menu, "items-changed",
+                                   G_CALLBACK (mousepad_application_update_menu), section, 0);
+        }
+      /* section GMenuItem without a share id: go ahead recursively */
+      else if (section != NULL)
+        mousepad_application_set_shared_menu_parts (application, section);
+      /* real GMenuItem */
+      else
+        {
+          /* set the menu item when shared */
+          if ((value = g_menu_model_get_item_attribute_value (model, n, "item-share-id",
+                                                              G_VARIANT_TYPE_STRING)))
+            {
+              share_id = g_variant_get_string (value, NULL);
+              g_variant_unref (value);
+
+              shared_item = G_MENU_MODEL (gtk_application_get_menu_by_id (
+                                            GTK_APPLICATION (application), share_id));
+              mousepad_object_set_data (model, g_intern_string (share_id), GINT_TO_POINTER (n));
+              mousepad_application_update_menu_item (shared_item, 0, 0, 1, model);
+
+              g_signal_connect_object (shared_item, "items-changed",
+                                       G_CALLBACK (mousepad_application_update_menu_item), model, 0);
+            }
+
+          /* submenu GMenuItem with a share id: insert the shared menu */
+          if ((submodel = g_menu_model_get_item_link (model, n, G_MENU_LINK_SUBMENU))
+              && ((value = g_menu_model_get_item_attribute_value (model, n, "submenu-share-id",
+                                                                  G_VARIANT_TYPE_STRING))))
+            {
+              share_id = g_variant_get_string (value, NULL);
+              g_variant_unref (value);
+
+              shared_menu = G_MENU_MODEL (gtk_application_get_menu_by_id (
+                                            GTK_APPLICATION (application), share_id));
+              mousepad_application_update_menu (shared_menu, 0, 0,
+                                                g_menu_model_get_n_items (shared_menu), submodel);
+
+              g_signal_connect_object (shared_menu, "items-changed",
+                                       G_CALLBACK (mousepad_application_update_menu), submodel, 0);
+            }
+          /* submenu GMenuItem without a share id: go ahead recursively */
+          else if (submodel != NULL)
+            mousepad_application_set_shared_menu_parts (application, submodel);
+        }
+    }
+}
+
+
+
 static void
 mousepad_application_create_languages_menu (MousepadApplication *application)
 {
@@ -852,7 +1187,7 @@ mousepad_application_create_languages_menu (MousepadApplication *application)
   /* get the empty "Filetype" submenu and populate it */
   menu = gtk_application_get_menu_by_id (GTK_APPLICATION (application), "document.filetype.list");
 
-  sections = mousepad_util_get_sorted_section_names ();
+  sections = mousepad_util_get_sorted_language_sections ();
 
   for (iter_sect = sections; iter_sect != NULL; iter_sect = g_slist_next (iter_sect))
     {
@@ -862,7 +1197,7 @@ mousepad_application_create_languages_menu (MousepadApplication *application)
       item = g_menu_item_new_submenu (label, G_MENU_MODEL (submenu));
 
       /* set tooltip and append menu item */
-      tooltip = (gchar*) iter_sect->data;
+      tooltip = iter_sect->data;
       g_menu_item_set_attribute_value (item, "tooltip", g_variant_new_string (tooltip));
       g_menu_append_item (menu, item);
       g_object_unref (item);
@@ -878,7 +1213,7 @@ mousepad_application_create_languages_menu (MousepadApplication *application)
           item = g_menu_item_new (label, action_name);
 
           /* set tooltip */
-          tooltip = g_strdup_printf ("%s/%s", (gchar*) iter_sect->data, label);
+          tooltip = g_strdup_printf ("%s/%s", (gchar *) iter_sect->data, label);
           g_menu_item_set_attribute_value (item, "tooltip", g_variant_new_string (tooltip));
 
           /* append menu item */
@@ -911,13 +1246,13 @@ mousepad_application_create_style_schemes_menu (MousepadApplication *application
   /* get the empty "Color Scheme" submenu and populate it */
   menu = gtk_application_get_menu_by_id (GTK_APPLICATION (application), "view.color-scheme.list");
 
-  schemes = mousepad_util_style_schemes_get_sorted ();
+  schemes = mousepad_util_get_sorted_style_schemes ();
 
   for (iter = schemes; iter != NULL; iter = g_slist_next (iter))
     {
       /* create menu item */
       label = gtk_source_style_scheme_get_id (iter->data);
-      action_name = g_strconcat ("win.view.color-scheme('", label, "')", NULL);
+      action_name = g_strconcat ("app.preferences.view.color-scheme('", label, "')", NULL);
       label = gtk_source_style_scheme_get_name (iter->data);
       item = g_menu_item_new (label, action_name);
 
@@ -952,3 +1287,77 @@ mousepad_application_action_quit (GSimpleAction *action,
 {
   g_application_quit (G_APPLICATION (data));
 }
+
+
+
+static void
+mousepad_application_toggle_activate (GSimpleAction *action,
+                                      GVariant      *parameter,
+                                      gpointer       data)
+{
+  gboolean state;
+
+  /* save the setting */
+  state = ! g_variant_get_boolean (g_action_get_state (G_ACTION (action)));
+  mousepad_setting_set_boolean (g_action_get_name (G_ACTION (action)), state);
+}
+
+
+
+static void
+mousepad_application_radio_activate (GSimpleAction *action,
+                                     GVariant      *parameter,
+                                     gpointer       data)
+{
+  /* save the setting */
+  mousepad_setting_set_variant (g_action_get_name (G_ACTION (action)), parameter);
+}
+
+
+
+static void
+mousepad_application_action_update (MousepadApplication *application,
+                                    gchar               *key,
+                                    GSettings           *settings)
+{
+  GVariant *state;
+  gchar    *schema_id, *action_name;
+
+  /* retrieve the action name from the setting schema id */
+  g_object_get (settings, "schema_id", &schema_id, NULL);
+  action_name = g_strdup_printf ("%s.%s", schema_id + MOUSEPAD_ID_LEN + 1, key);
+
+  /* request for the action state to be changed according to the setting */
+  state = mousepad_setting_get_variant (action_name);
+  g_action_group_change_action_state (G_ACTION_GROUP (application), action_name, state);
+
+  /* cleanup */
+  g_free (schema_id);
+  g_free (action_name);
+  g_variant_unref (state);
+}
+
+
+
+/*
+ * This job is done only once here, when whitespace location settings are updated,
+ * then all the views update their own location flags from those of the application.
+ */
+static void
+mousepad_application_action_whitespace (GSimpleAction *action,
+                                        GVariant      *state,
+                                        gpointer       data)
+{
+  GtkSourceSpaceLocationFlags flags, flag;
+
+  /* change action state */
+  g_simple_action_set_state (action, state);
+
+  /* update the space location flags */
+  flags = MOUSEPAD_APPLICATION (data)->space_location_flags;
+  flag = GPOINTER_TO_UINT (mousepad_object_get_data (action, "flag"));
+  g_variant_get_boolean (state) ? (flags |= flag) : (flags &= ~flag);
+
+  /* set the application property */
+  g_object_set (data, "space-location", flags, NULL);
+}
