@@ -18,8 +18,6 @@
 #include <mousepad/mousepad-dialogs.h>
 #include <mousepad/mousepad-util.h>
 
-#include <gtksourceview/gtksource.h>
-
 
 
 void
@@ -41,7 +39,7 @@ mousepad_dialogs_show_about (GtkWindow *parent)
                          "copyright", "Copyright \xc2\xa9 2005-2020 - the Mousepad developers",
                          "destroy-with-parent", TRUE,
                          "license-type", GTK_LICENSE_GPL_2_0,
-                         "logo-icon-name", "org.xfce.mousepad",
+                         "logo-icon-name", MOUSEPAD_ID,
                          "program-name", PACKAGE_NAME,
                          "version", PACKAGE_VERSION,
                          "translator-credits", _("translator-credits"),
@@ -83,40 +81,16 @@ mousepad_dialogs_show_help (GtkWindow   *parent,
                             const gchar *page,
                             const gchar *offset)
 {
-#if !GTK_CHECK_VERSION (3, 22, 0)
-  GdkScreen   *screen;
-#endif
   GError      *error = NULL;
   const gchar *uri = "https://docs.xfce.org/apps/mousepad/start";
 
   /* try to run the documentation browser */
-#if GTK_CHECK_VERSION (3, 22, 0)
   if (!gtk_show_uri_on_window (parent, uri, gtk_get_current_event_time (), &error))
     {
       /* display an error message to the user */
       mousepad_dialogs_show_error (parent, error, _("Failed to open the documentation browser"));
       g_error_free (error);
     }
-#else
-
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-
-  /* get screen */
-  if (G_LIKELY (parent))
-    screen = gtk_widget_get_screen (GTK_WIDGET (parent));
-  else
-    screen = gdk_screen_get_default ();
-
-  if (!gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error))
-    {
-      /* display an error message to the user */
-      mousepad_dialogs_show_error (parent, error, _("Failed to open the documentation browser"));
-      g_error_free (error);
-    }
-
-G_GNUC_END_IGNORE_DEPRECATIONS
-
-#endif
 }
 
 
@@ -171,7 +145,7 @@ mousepad_dialogs_go_to_line_changed (GtkSpinButton *line_spin,
   g_return_if_fail (GTK_IS_SPIN_BUTTON (col_spin));
 
   /* get the text buffer */
-  buffer = mousepad_object_get_data (G_OBJECT (col_spin), "buffer");
+  buffer = mousepad_object_get_data (col_spin, "buffer");
 
   /* debug check */
   g_return_if_fail (GTK_IS_TEXT_BUFFER (buffer));
@@ -265,7 +239,7 @@ mousepad_dialogs_go_to (GtkWindow     *parent,
 
   col_spin = gtk_spin_button_new_with_range (0, 0, 1);
   gtk_entry_set_activates_default (GTK_ENTRY (col_spin), TRUE);
-  mousepad_object_set_data (G_OBJECT (col_spin), "buffer", buffer);
+  mousepad_object_set_data (col_spin, "buffer", buffer);
   gtk_box_pack_start (GTK_BOX (hbox), col_spin, FALSE, FALSE, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), col_spin);
   gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (col_spin), TRUE);
@@ -273,7 +247,8 @@ mousepad_dialogs_go_to (GtkWindow     *parent,
   gtk_widget_show (col_spin);
 
   /* signal to monitor column number */
-  g_signal_connect (G_OBJECT (line_spin), "value-changed", G_CALLBACK (mousepad_dialogs_go_to_line_changed), col_spin);
+  g_signal_connect (line_spin, "value-changed",
+                    G_CALLBACK (mousepad_dialogs_go_to_line_changed), col_spin);
   gtk_spin_button_set_value (GTK_SPIN_BUTTON (line_spin), line);
 
   /* run the dialog */
@@ -295,7 +270,7 @@ mousepad_dialogs_go_to (GtkWindow     *parent,
     }
 
   /* release size group */
-  g_object_unref (G_OBJECT (size_group));
+  g_object_unref (size_group);
 
   /* destroy the dialog */
   gtk_widget_destroy (dialog);
