@@ -16,30 +16,16 @@
 
 #include <mousepad/mousepad-private.h>
 #include <mousepad/mousepad-file.h>
-#if !GLIB_CHECK_VERSION (2, 52, 0)
-#include <mousepad/mousepad-util.h>
-#endif
 
 #include <glib/gstdio.h>
 
-#include <gtksourceview/gtksource.h>
-
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
 
+#ifdef HAVE_ERRNO_H
 #include <errno.h>
-
-
+#endif
 
 enum
 {
@@ -167,7 +153,7 @@ mousepad_file_finalize (GObject *object)
   g_free (file->filename);
 
   /* release the reference from the buffer */
-  g_object_unref (G_OBJECT (file->buffer));
+  g_object_unref (file->buffer);
 
   (*G_OBJECT_CLASS (mousepad_file_parent_class)->finalize) (object);
 }
@@ -253,7 +239,7 @@ mousepad_file_new (GtkTextBuffer *buffer)
   file = g_object_new (MOUSEPAD_TYPE_FILE, NULL);
 
   /* set the buffer */
-  file->buffer = GTK_TEXT_BUFFER (g_object_ref (G_OBJECT (buffer)));
+  file->buffer = GTK_TEXT_BUFFER (g_object_ref (buffer));
 
   return file;
 }
@@ -283,7 +269,7 @@ mousepad_file_set_filename (MousepadFile *file,
       file->temporary = FALSE;
 
       /* send a signal that the name has been changed */
-      g_signal_emit (G_OBJECT (file), file_signals[FILENAME_CHANGED], 0, file->filename);
+      g_signal_emit (file, file_signals[FILENAME_CHANGED], 0, file->filename);
     }
   /* toggle filename state */
   else
@@ -346,7 +332,7 @@ mousepad_file_set_read_only (MousepadFile *file,
       file->readonly = readonly;
 
       /* emit signal */
-      g_signal_emit (G_OBJECT (file), file_signals[READONLY_CHANGED], 0, readonly);
+      g_signal_emit (file, file_signals[READONLY_CHANGED], 0, readonly);
     }
 }
 
@@ -559,7 +545,7 @@ mousepad_file_open (MousepadFile  *file,
 
           /* leave when the contents is not utf-8 valid */
           if (G_LIKELY (file->encoding == MOUSEPAD_ENCODING_UTF_8)
-              && g_utf8_validate_len (contents, file_size, &end) == FALSE)
+              && g_utf8_validate (contents, file_size, &end) == FALSE)
             {
               /* set return value */
               retval = ERROR_NOT_UTF8_VALID;
@@ -789,9 +775,9 @@ mousepad_file_save (MousepadFile  *file,
               memmove (contents + 3, contents, length + 1);
 
               /* write an utf-8 bom at the start of the contents */
-              contents[0] = (guchar) 0xef;
-              contents[1] = (guchar) 0xbb;
-              contents[2] = (guchar) 0xbf;
+              contents[0] = 0xef;
+              contents[1] = 0xbb;
+              contents[2] = 0xbf;
 
               /* increase the length */
               length += 3;
