@@ -123,7 +123,8 @@ mousepad_statusbar_init (MousepadStatusbar *statusbar)
   gtk_box_pack_start (GTK_BOX (box), ebox, FALSE, TRUE, 0);
   gtk_event_box_set_visible_window (GTK_EVENT_BOX (ebox), FALSE);
   gtk_widget_set_tooltip_text (ebox, _("Choose a filetype"));
-  g_signal_connect (G_OBJECT (ebox), "button-press-event", G_CALLBACK (mousepad_statusbar_filetype_clicked), statusbar);
+  g_signal_connect (ebox, "button-press-event",
+                    G_CALLBACK (mousepad_statusbar_filetype_clicked), statusbar);
   gtk_widget_show (ebox);
 
   /* language/filetype */
@@ -151,7 +152,8 @@ mousepad_statusbar_init (MousepadStatusbar *statusbar)
   gtk_box_pack_start (GTK_BOX (box), ebox, FALSE, TRUE, 0);
   gtk_event_box_set_visible_window (GTK_EVENT_BOX (ebox), FALSE);
   gtk_widget_set_tooltip_text (ebox, _("Toggle the overwrite mode"));
-  g_signal_connect (G_OBJECT (ebox), "button-press-event", G_CALLBACK (mousepad_statusbar_overwrite_clicked), statusbar);
+  g_signal_connect (ebox, "button-press-event",
+                    G_CALLBACK (mousepad_statusbar_overwrite_clicked), statusbar);
   gtk_widget_show (ebox);
 
   /* overwrite label */
@@ -177,7 +179,7 @@ mousepad_statusbar_overwrite_clicked (GtkWidget         *widget,
   statusbar->overwrite_enabled = !statusbar->overwrite_enabled;
 
   /* send the signal */
-  g_signal_emit (G_OBJECT (statusbar), statusbar_signals[ENABLE_OVERWRITE], 0, statusbar->overwrite_enabled);
+  g_signal_emit (statusbar, statusbar_signals[ENABLE_OVERWRITE], 0, statusbar->overwrite_enabled);
 
   return TRUE;
 }
@@ -203,29 +205,15 @@ mousepad_statusbar_filetype_clicked (GtkWidget         *widget,
   /* get the languages menu from the window */
   window = gtk_widget_get_ancestor (GTK_WIDGET (statusbar), MOUSEPAD_TYPE_WINDOW);
   menu = GTK_MENU (mousepad_window_get_languages_menu (MOUSEPAD_WINDOW (window)));
-  g_warn_if_fail (GTK_IS_MENU (menu));
 
   /* get the number of items in the menu */
   children = gtk_container_get_children (GTK_CONTAINER (menu));
   n_children = g_list_length (children);
   g_list_free (children);
 
-  /* make sure there's at least one item in the menu */
+  /* make sure there's at least one item in the menu to show it */
   if (n_children)
-    {
-      /* show the menu */
-#if GTK_CHECK_VERSION (3, 22, 0)
-      gtk_menu_popup_at_pointer (menu, (GdkEvent*) event);
-#else
-
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-
-      gtk_menu_popup (menu, NULL, NULL, NULL, NULL, event->button, event->time);
-
-G_GNUC_END_IGNORE_DEPRECATIONS
-
-#endif
-    }
+    gtk_menu_popup_at_pointer (menu, (GdkEvent *) event);
 
   return TRUE;
 }
