@@ -17,10 +17,15 @@
 #include <mousepad/mousepad-private.h>
 #include <mousepad/mousepad-util.h>
 
+#include <xfconf/xfconf.h>
+
+#ifdef HAVE_MATH_H
 #include <math.h>
+#endif
 
 
 
+#define DEFAULT_FONT "Monospace 10"
 #define FONT_FAMILY  "font-family"
 #define FONT_STYLE   "font-style"
 #define FONT_VARIANT "font-variant"
@@ -47,7 +52,7 @@ mousepad_util_iter_word_characters (const GtkTextIter *iter)
 
 
 
-gboolean
+static gboolean
 mousepad_util_iter_starts_word (const GtkTextIter *iter)
 {
   GtkTextIter prev;
@@ -72,7 +77,7 @@ mousepad_util_iter_starts_word (const GtkTextIter *iter)
 
 
 
-gboolean
+static gboolean
 mousepad_util_iter_ends_word (const GtkTextIter *iter)
 {
   if (!gtk_text_iter_ends_word (iter))
@@ -397,7 +402,7 @@ mousepad_util_entry_error (GtkWidget *widget,
   g_return_if_fail (GTK_IS_WIDGET (widget));
 
   /* get the current error state */
-  pointer = mousepad_object_get_data (G_OBJECT (widget), "error-state");
+  pointer = mousepad_object_get_data (widget, "error-state");
 
   /* only change the state when really needed to avoid multiple widget calls */
   if (GPOINTER_TO_INT (pointer) != error)
@@ -410,7 +415,7 @@ mousepad_util_entry_error (GtkWidget *widget,
         gtk_style_context_remove_class (gtk_widget_get_style_context (widget), GTK_STYLE_CLASS_ERROR);
 
       /* set the new state */
-      mousepad_object_set_data (G_OBJECT (widget), "error-state", GINT_TO_POINTER (error));
+      mousepad_object_set_data (widget, "error-state", GINT_TO_POINTER (error));
     }
 }
 
@@ -429,7 +434,7 @@ mousepad_util_dialog_create_header (GtkDialog   *dialog,
 
   /* remove the main vbox */
   dialog_vbox = gtk_bin_get_child (GTK_BIN (dialog));
-  g_object_ref (G_OBJECT (dialog_vbox));
+  g_object_ref (dialog_vbox);
   gtk_container_remove (GTK_CONTAINER (dialog), dialog_vbox);
 
   /* add a new vbox to the main window */
@@ -476,7 +481,7 @@ mousepad_util_dialog_create_header (GtkDialog   *dialog,
 
   /* add the main dialog box to the new vbox */
   gtk_box_pack_start (GTK_BOX (vbox), dialog_vbox, TRUE, TRUE, 0);
-  g_object_unref (G_OBJECT (dialog_vbox));
+  g_object_unref (dialog_vbox);
 }
 
 
@@ -602,8 +607,8 @@ mousepad_util_get_save_location (const gchar *relpath,
           if (g_mkdir_with_parents (dirname, 0700) == -1)
             {
               /* show warning to the user */
-              g_critical (_("Unable to create base directory \"%s\". "
-                            "Saving to file \"%s\" will be aborted."), dirname, filename);
+              g_critical ("Unable to create base directory \"%s\". "
+                          "Saving to file \"%s\" will be aborted.", dirname, filename);
 
               /* don't return a filename, to avoid problems */
               g_free (filename);
@@ -648,7 +653,7 @@ mousepad_util_save_key_file (GKeyFile    *keyfile,
       print_error:
 
       /* print error */
-      g_critical (_("Failed to store the preferences to \"%s\": %s"), filename, error->message);
+      g_critical ("Failed to store the preferences to \"%s\": %s", filename, error->message);
 
       /* cleanup */
       g_error_free (error);
@@ -660,191 +665,12 @@ mousepad_util_save_key_file (GKeyFile    *keyfile,
 
 
 
-GType
-mousepad_util_search_flags_get_type (void)
-{
-  static GType type = G_TYPE_NONE;
-
-  if (G_UNLIKELY (type == G_TYPE_NONE))
-    {
-      /* use empty values table */
-      static const GFlagsValue values[] =
-      {
-          { 0, NULL, NULL }
-      };
-
-      /* register the type */
-      type = g_flags_register_static (I_("MousepadSearchFlags"), values);
-      }
-
-  return type;
-}
-
-
-
-gint
-mousepad_util_search (GtkSourceSearchContext *search_context,
-                      const gchar            *string,
-                      const gchar            *replace,
-                      MousepadSearchFlags     flags)
-{
-  GtkSourceSearchSettings *search_settings;
-  GtkTextBuffer           *buffer, *selection_buffer = NULL;
-  GtkTextIter              start, end, iter, bstart, fend, biter, fiter;
-  gchar                   *selected_text;
-  gint                     counter = 0;
-  gboolean                 found;
-
-  g_return_val_if_fail (GTK_SOURCE_IS_SEARCH_CONTEXT (search_context), -1);
-  g_return_val_if_fail (string != NULL && g_utf8_validate (string, -1, NULL), -1);
-  g_return_val_if_fail (replace == NULL || g_utf8_validate (replace, -1, NULL), -1);
-
-  /* freeze buffer notifications */
-  buffer = GTK_TEXT_BUFFER (gtk_source_search_context_get_buffer (search_context));
-  g_object_freeze_notify (G_OBJECT (buffer));
-
-  /* get the search iters */
-  if (flags & MOUSEPAD_SEARCH_FLAGS_ITER_SEL_START)
-    gtk_text_buffer_get_selection_bounds (buffer, &iter, NULL);
-  else
-    gtk_text_buffer_get_selection_bounds (buffer, NULL, &iter);
-
-  /* substitute a virtual selection buffer to the real buffer and work inside */
-  if (flags & MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION)
-    {
-      selection_buffer = GTK_TEXT_BUFFER (gtk_source_buffer_new (NULL));
-      gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
-      selected_text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-      gtk_text_buffer_set_text (selection_buffer, selected_text, -1);
-      search_context = gtk_source_search_context_new (GTK_SOURCE_BUFFER (selection_buffer), NULL);
-      gtk_text_buffer_get_start_iter (selection_buffer, &iter);
-    }
-
-  /* set the search context settings */
-  search_settings = gtk_source_search_context_get_settings (search_context);
-  gtk_source_search_settings_set_search_text (search_settings, string);
-  gtk_source_search_settings_set_case_sensitive (search_settings,
-                                                 flags & MOUSEPAD_SEARCH_FLAGS_MATCH_CASE);
-  gtk_source_search_settings_set_at_word_boundaries (search_settings,
-                                                     flags & MOUSEPAD_SEARCH_FLAGS_WHOLE_WORD);
-  gtk_source_search_settings_set_wrap_around (search_settings,
-                                              flags & MOUSEPAD_SEARCH_FLAGS_WRAP_AROUND);
-  gtk_source_search_settings_set_regex_enabled (search_settings,
-                                                flags & MOUSEPAD_SEARCH_FLAGS_ENABLE_REGEX);
-
-  /* search the string */
-  if (flags & MOUSEPAD_SEARCH_FLAGS_DIR_BACKWARD)
-    found = gtk_source_search_context_backward2 (search_context, &iter, &start, &end, NULL);
-  else
-    found = gtk_source_search_context_forward2 (search_context, &iter, &start, &end, NULL);
-
-  /* set the counter, ensuring the buffer is fully scanned if needed (searching in both
-   * directions leads faster to a full scan) */
-  if (! (flags & MOUSEPAD_SEARCH_FLAGS_ENTIRE_AREA) || *string == '\0')
-    counter = found;
-  else
-    {
-      counter = gtk_source_search_context_get_occurrences_count (search_context);
-      if (counter == -1)
-        {
-          gtk_source_search_settings_set_wrap_around (search_settings, TRUE);
-          if (! found)
-            {
-              start = iter;
-              end = iter;
-            }
-          biter = start;
-          fiter = end;
-          do
-            {
-              gtk_source_search_context_backward2 (search_context, &biter, &bstart, NULL, NULL);
-              gtk_source_search_context_forward2 (search_context, &fiter, NULL, &fend, NULL);
-              counter = gtk_source_search_context_get_occurrences_count (search_context);
-              biter = bstart;
-              fiter = fend;
-            }
-          while (counter == -1 && ! gtk_text_iter_equal (&biter, &start)
-                               && ! gtk_text_iter_equal (&fiter, &end));
-          gtk_source_search_settings_set_wrap_around (search_settings,
-                                                      flags & MOUSEPAD_SEARCH_FLAGS_WRAP_AROUND);
-        }
-    }
-
-  /* handle the action */
-  if (found && (flags & MOUSEPAD_SEARCH_FLAGS_ACTION_SELECT)
-      && ! (flags & MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION))
-    gtk_text_buffer_select_range (buffer, &start, &end);
-  else if (flags & MOUSEPAD_SEARCH_FLAGS_ACTION_REPLACE)
-    {
-      if (found && ! (flags & MOUSEPAD_SEARCH_FLAGS_ENTIRE_AREA))
-        {
-          /* replace selected occurrence */
-          gtk_source_search_context_replace2 (search_context, &start, &end, replace, -1, NULL);
-
-          /* select next occurrence */
-          flags |= MOUSEPAD_SEARCH_FLAGS_ACTION_SELECT;
-          flags &= ~MOUSEPAD_SEARCH_FLAGS_ACTION_REPLACE;
-          counter = mousepad_util_search (search_context, string, NULL, flags);
-        }
-      else if (counter > 0 && (flags & MOUSEPAD_SEARCH_FLAGS_ENTIRE_AREA))
-        {
-          gtk_source_search_context_replace_all (search_context, replace, -1, NULL);
-          if (flags & MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION)
-            {
-              /* replace all occurrences in the virtual selection buffer */
-              gtk_source_search_context_replace_all (search_context, replace, -1, NULL);
-              gtk_text_buffer_get_bounds (selection_buffer, &start, &end);
-              selected_text = gtk_text_buffer_get_text (selection_buffer, &start, &end, FALSE);
-
-              /* replace selection in the real buffer by the text in the virtual selection buffer */
-              gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
-              gtk_text_buffer_begin_user_action (buffer);
-              gtk_text_buffer_delete (buffer, &start, &end);
-              gtk_text_buffer_insert (buffer, &start, selected_text, -1);
-              gtk_text_buffer_end_user_action (buffer);
-            }
-        }
-    }
-  else if (! (flags & MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION))
-    gtk_text_buffer_place_cursor (buffer, &iter);
-
-  /* thawn buffer notifications */
-  g_object_thaw_notify (G_OBJECT (buffer));
-
-  /* cleanup */
-  if (flags & MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION)
-    {
-      g_object_unref (G_OBJECT (selection_buffer));
-      g_object_unref (G_OBJECT (search_context));
-      g_free (selected_text);
-    }
-
-  return counter;
-}
-
-
-
-GIcon *
-mousepad_util_icon_for_mime_type (const gchar *mime_type)
-{
-  gchar *content_type;
-  GIcon *icon = NULL;
-
-  g_return_val_if_fail (mime_type != NULL, NULL);
-
-  content_type = g_content_type_from_mime_type (mime_type);
-  if (content_type != NULL)
-    icon = g_content_type_get_icon (content_type);
-
-  return icon;
-}
-
-
-
 static void
 mousepad_util_container_foreach_counter (GtkWidget *widget,
-                                         guint     *n_children)
+                                         gpointer   data)
 {
+  guint *n_children = (guint *) data;
+
   *n_children = *n_children + 1;
 }
 
@@ -858,7 +684,7 @@ mousepad_util_container_has_children (GtkContainer *container)
   g_return_val_if_fail (GTK_IS_CONTAINER (container), FALSE);
 
   gtk_container_foreach (container,
-                         (GtkCallback) mousepad_util_container_foreach_counter,
+                         mousepad_util_container_foreach_counter,
                          &n_children);
 
   return (n_children > 0);
@@ -906,7 +732,7 @@ mousepad_util_container_move_children (GtkContainer *source,
 
 static gint
 mousepad_util_style_schemes_name_compare (gconstpointer a,
-                                                  gconstpointer b)
+                                          gconstpointer b)
 {
   const gchar *name_a, *name_b;
 
@@ -947,7 +773,7 @@ mousepad_util_get_style_schemes (void)
 
 
 GSList *
-mousepad_util_style_schemes_get_sorted (void)
+mousepad_util_get_sorted_style_schemes (void)
 {
   return g_slist_sort (mousepad_util_get_style_schemes (),
                        mousepad_util_style_schemes_name_compare);
@@ -956,7 +782,7 @@ mousepad_util_style_schemes_get_sorted (void)
 
 
 GSList *
-mousepad_util_get_sorted_section_names (void)
+mousepad_util_get_sorted_language_sections (void)
 {
   GSList                   *list = NULL;
   const gchar *const       *languages;
@@ -996,7 +822,7 @@ mousepad_util_get_sorted_section_names (void)
 
 static gint
 mousepad_util_languages_name_compare (gconstpointer a,
-                                              gconstpointer b)
+                                      gconstpointer b)
 {
   const gchar *name_a, *name_b;
 
@@ -1045,81 +871,24 @@ mousepad_util_get_sorted_languages_for_section (const gchar *section)
       languages++;
     }
 
-  return g_slist_sort (list, (GCompareFunc) mousepad_util_languages_name_compare);
+  return g_slist_sort (list, mousepad_util_languages_name_compare);
 }
 
 
 
-#if !GLIB_CHECK_VERSION (2, 52, 0)
-/*
- * Copied from GLib 2.66.0:
- * https://gitlab.gnome.org/GNOME/glib/-/blob/c2c12e42920d6e06c23c87398996827e53c1fc72/glib/gutf8.c#L1791
- */
-/**
- * g_utf8_make_valid:
- * @str: string to coerce into UTF-8
- * @len: the maximum length of @str to use, in bytes. If @len < 0,
- *     then the string is nul-terminated.
- *
- * If the provided string is valid UTF-8, return a copy of it. If not,
- * return a copy in which bytes that could not be interpreted as valid Unicode
- * are replaced with the Unicode replacement character (U+FFFD).
- *
- * For example, this is an appropriate function to use if you have received
- * a string that was incorrectly declared to be UTF-8, and you need a valid
- * UTF-8 version of it that can be logged or displayed to the user, with the
- * assumption that it is close enough to ASCII or UTF-8 to be mostly
- * readable as-is.
- *
- * Returns: (transfer full): a valid UTF-8 string whose content resembles @str
- *
- * Since: 2.52
- */
 gchar *
-g_utf8_make_valid (const gchar *str,
-                   gssize       len)
+mousepad_util_get_default_font (void)
 {
-  GString *string;
-  const gchar *remainder, *invalid;
-  gsize remaining_bytes, valid_bytes;
+  gchar *font;
 
-  g_return_val_if_fail (str != NULL, NULL);
+  /* get the default font from xfconf */
+  font = xfconf_channel_get_string (xfconf_channel_get ("xsettings"),
+                                    "/Gtk/MonospaceFontName", NULL);
+  if (G_UNLIKELY (font == NULL))
+    font = g_strdup (DEFAULT_FONT);
 
-  if (len < 0)
-    len = strlen (str);
-
-  string = NULL;
-  remainder = str;
-  remaining_bytes = len;
-
-  while (remaining_bytes != 0)
-    {
-      if (g_utf8_validate (remainder, remaining_bytes, &invalid))
-        break;
-      valid_bytes = invalid - remainder;
-
-      if (string == NULL)
-        string = g_string_sized_new (remaining_bytes);
-
-      g_string_append_len (string, remainder, valid_bytes);
-      /* append U+FFFD REPLACEMENT CHARACTER */
-      g_string_append (string, "\357\277\275");
-
-      remaining_bytes -= valid_bytes + 1;
-      remainder = invalid + 1;
-    }
-
-  if (string == NULL)
-    return g_strndup (str, len);
-
-  g_string_append_len (string, remainder, remaining_bytes);
-  g_string_append_c (string, '\0');
-
-  g_assert (g_utf8_validate (string->str, -1, NULL));
-
-  return g_string_free (string, FALSE);
+  return font;
 }
-#endif
 
 
 
@@ -1128,7 +897,7 @@ g_utf8_make_valid (const gchar *str,
  * https://gitlab.gnome.org/GNOME/gedit/-/blob/21fac3f0c87db0db104d7af7eaeb6f63d8216a14/gedit/gedit-pango.c#L37
  */
 /**
- * mousepad_pango_font_description_to_css:
+ * mousepad_util_pango_font_description_to_css:
  *
  * This function will generate CSS suitable for Gtk's CSS engine
  * based on the properties of the #PangoFontDescription.
@@ -1137,7 +906,7 @@ g_utf8_make_valid (const gchar *str,
  *    CSS describing the font description.
  */
 gchar *
-mousepad_pango_font_description_to_css (const PangoFontDescription *font_desc)
+mousepad_util_pango_font_description_to_css (const PangoFontDescription *font_desc)
 {
   PangoFontMask  mask;
   GString       *str;
