@@ -19,12 +19,8 @@
 #include <mousepad/mousepad-util.h>
 #include <mousepad/mousepad-view.h>
 
-#include <gtksourceview/gtksource.h>
-
-#include <xfconf/xfconf.h>
 
 
-#define MOUSEPAD_VIEW_DEFAULT_FONT "Monospace 10"
 #define mousepad_view_get_buffer(view) (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)))
 
 
@@ -34,10 +30,6 @@ static void      mousepad_view_set_property                  (GObject
                                                               guint               prop_id,
                                                               const GValue       *value,
                                                               GParamSpec         *pspec);
-static void      mousepad_view_get_property                  (GObject            *object,
-                                                              guint               prop_id,
-                                                              GValue             *value,
-                                                              GParamSpec         *pspec);
 static gboolean  mousepad_view_key_press_event               (GtkWidget          *widget,
                                                               GdkEventKey        *event);
 static void      mousepad_view_indent_increase               (MousepadView       *view,
@@ -55,7 +47,20 @@ static void      mousepad_view_transpose_lines               (GtkTextBuffer
                                                               GtkTextIter         *end_iter);
 static void      mousepad_view_transpose_words               (GtkTextBuffer       *buffer,
                                                               GtkTextIter         *iter);
-static void      mousepad_view_update_font                   (MousepadView        *view);
+static void      mousepad_view_set_font                      (MousepadView        *view,
+                                                              const gchar         *font);
+static void      mousepad_view_set_show_whitespace           (MousepadView        *view,
+                                                              gboolean             show);
+static void      mousepad_view_set_space_location_flags      (MousepadView        *view,
+                                                              gulong               flags);
+static void      mousepad_view_set_show_line_endings         (MousepadView        *view,
+                                                              gboolean             show);
+static void      mousepad_view_set_color_scheme              (MousepadView        *view,
+                                                              const gchar         *color_scheme);
+static void      mousepad_view_set_word_wrap                 (MousepadView        *view,
+                                                              gboolean             enabled);
+static void      mousepad_view_set_match_braces              (MousepadView        *view,
+                                                              gboolean             enabled);
 
 
 
@@ -68,37 +73,12 @@ struct _MousepadView
 {
   GtkSourceView         __parent__;
 
-  /* the selection style tag */
-  GtkTextTag           *selection_tag;
-
-  /* list with all the selection marks */
-  GSList               *selection_marks;
-
-  /* selection timeout id */
-  guint                 selection_timeout_id;
-
-  /* coordinates for the selection */
-  gint                  selection_start_x;
-  gint                  selection_start_y;
-  gint                  selection_end_x;
-  gint                  selection_end_y;
-
-  /* length of the selection (-1 = zerro width selection, 0 = no selection) */
-  gint                  selection_length;
-
-  /* if the selection is in editing mode */
-  guint                 selection_editing : 1;
-
-  /* the font used in the view */
-  PangoFontDescription *font_desc;
-
-  /* whitespace visualization */
-  gboolean              show_whitespace;
-  gboolean              show_line_endings;
-
-  gchar                *color_scheme;
-
-  gboolean              match_braces;
+  /* property related */
+  gboolean                     show_whitespace;
+  GtkSourceSpaceLocationFlags  space_location_flags;
+  gboolean                     show_line_endings;
+  gchar                       *color_scheme;
+  gboolean                     match_braces;
 };
 
 
@@ -106,8 +86,9 @@ struct _MousepadView
 enum
 {
   PROP_0,
-  PROP_FONT_NAME,
+  PROP_FONT,
   PROP_SHOW_WHITESPACE,
+  PROP_SPACE_LOCATION,
   PROP_SHOW_LINE_ENDINGS,
   PROP_COLOR_SCHEME,
   PROP_WORD_WRAP,
@@ -130,64 +111,44 @@ mousepad_view_class_init (MousepadViewClass *klass)
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = mousepad_view_finalize;
   gobject_class->set_property = mousepad_view_set_property;
-  gobject_class->get_property = mousepad_view_get_property;
 
   widget_class = GTK_WIDGET_CLASS (klass);
   widget_class->key_press_event = mousepad_view_key_press_event;
 
-  g_object_class_install_property (
-    gobject_class,
-    PROP_FONT_NAME,
-    g_param_spec_string ("font-name",
-                         "FontName",
-                         "The name of the font to use in the view",
-                         MOUSEPAD_VIEW_DEFAULT_FONT,
-                         G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_FONT,
+    g_param_spec_string ("font", "Font", "The font to use in the view",
+                         NULL, G_PARAM_WRITABLE));
 
-  g_object_class_install_property (
-    gobject_class,
-    PROP_SHOW_WHITESPACE,
-    g_param_spec_boolean ("show-whitespace",
-                          "ShowWhitespace",
+  g_object_class_install_property (gobject_class, PROP_SHOW_WHITESPACE,
+    g_param_spec_boolean ("show-whitespace", "ShowWhitespace",
                           "Whether whitespace is visualized in the view",
-                          FALSE,
-                          G_PARAM_READWRITE));
+                          FALSE, G_PARAM_WRITABLE));
 
-  g_object_class_install_property (
-    gobject_class,
-    PROP_SHOW_LINE_ENDINGS,
-    g_param_spec_boolean ("show-line-endings",
-                          "ShowLineEndings",
+  g_object_class_install_property (gobject_class, PROP_SPACE_LOCATION,
+    g_param_spec_flags ("space-location", "SpaceLocation",
+                        "The space locations to show in the view",
+                        GTK_SOURCE_TYPE_SPACE_LOCATION_FLAGS, GTK_SOURCE_SPACE_LOCATION_ALL,
+                        G_PARAM_WRITABLE));
+
+  g_object_class_install_property (gobject_class, PROP_SHOW_LINE_ENDINGS,
+    g_param_spec_boolean ("show-line-endings", "ShowLineEndings",
                           "Whether line-endings are visualized in the view",
-                          FALSE,
-                          G_PARAM_READWRITE));
+                          FALSE, G_PARAM_WRITABLE));
 
-  g_object_class_install_property (
-    gobject_class,
-    PROP_COLOR_SCHEME,
-    g_param_spec_string ("color-scheme",
-                         "ColorScheme",
+  g_object_class_install_property (gobject_class, PROP_COLOR_SCHEME,
+    g_param_spec_string ("color-scheme", "ColorScheme",
                          "The id of the syntax highlighting color scheme to use",
-                         NULL,
-                         G_PARAM_READWRITE));
+                         NULL, G_PARAM_WRITABLE));
 
-  g_object_class_install_property (
-    gobject_class,
-    PROP_WORD_WRAP,
-    g_param_spec_boolean ("word-wrap",
-                          "WordWrap",
+  g_object_class_install_property (gobject_class, PROP_WORD_WRAP,
+    g_param_spec_boolean ("word-wrap", "WordWrap",
                           "Whether to virtually wrap long lines in the view",
-                          FALSE,
-                          G_PARAM_READWRITE));
+                          FALSE, G_PARAM_WRITABLE));
 
-  g_object_class_install_property (
-    gobject_class,
-    PROP_MATCH_BRACES,
-    g_param_spec_boolean ("match-braces",
-                          "MatchBraces",
+  g_object_class_install_property (gobject_class, PROP_MATCH_BRACES,
+    g_param_spec_boolean ("match-braces", "MatchBraces",
                           "Whether to highlight matching braces, parens, brackets, etc.",
-                          FALSE,
-                          G_PARAM_READWRITE));
+                          FALSE, G_PARAM_WRITABLE));
 }
 
 
@@ -198,7 +159,7 @@ mousepad_view_buffer_changed (MousepadView *view,
                               gpointer      user_data)
 {
   GtkSourceBuffer *buffer;
-  buffer = (GtkSourceBuffer*) gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+  buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
 
   if (GTK_SOURCE_IS_BUFFER (buffer))
     {
@@ -225,11 +186,21 @@ mousepad_view_buffer_changed (MousepadView *view,
 
 
 static void
-mousepad_view_use_default_font_setting_changed (MousepadView *view,
-                                                gchar        *key,
-                                                GSettings    *settings)
+mousepad_view_use_default_font (MousepadView *view)
 {
-  mousepad_view_update_font (view);
+  gchar *font;
+
+  /* default font is used, unbind from GSettings */
+  if (MOUSEPAD_SETTING_GET_BOOLEAN (USE_DEFAULT_FONT))
+    {
+      g_settings_unbind (view, "font");
+      font = mousepad_util_get_default_font ();
+      mousepad_view_set_font (view, font);
+      g_free (font);
+    }
+  /* default font is not used, bind to GSettings */
+  else
+    MOUSEPAD_SETTING_BIND (FONT, view, "font", G_SETTINGS_BIND_GET);
 }
 
 
@@ -237,32 +208,24 @@ mousepad_view_use_default_font_setting_changed (MousepadView *view,
 static void
 mousepad_view_init (MousepadView *view)
 {
-  /* initialize selection variables */
-  view->selection_timeout_id = 0;
-  view->selection_tag = NULL;
-  view->selection_marks = NULL;
-  view->selection_length = 0;
-  view->selection_editing = FALSE;
+  GApplication *application;
+
+  /* initialize properties variables */
+  view->show_whitespace = FALSE;
+  view->space_location_flags = GTK_SOURCE_SPACE_LOCATION_ALL;
+  view->show_line_endings = FALSE;
   view->color_scheme = g_strdup ("none");
-  view->font_desc = NULL;
   view->match_braces = FALSE;
 
   /* make sure any buffers set on the view get the color scheme applied to them */
-  g_signal_connect (view,
-                    "notify::buffer",
-                    G_CALLBACK (mousepad_view_buffer_changed),
-                    NULL);
-
-  /* reset drag coordinates */
-  view->selection_start_x = view->selection_end_x = -1;
-  view->selection_start_y = view->selection_end_y = -1;
+  g_signal_connect (view, "notify::buffer",
+                    G_CALLBACK (mousepad_view_buffer_changed), NULL);
 
   /* bind Gsettings */
 #define BIND_(setting, prop) \
-  MOUSEPAD_SETTING_BIND (setting, view, prop, G_SETTINGS_BIND_DEFAULT)
+  MOUSEPAD_SETTING_BIND (setting, view, prop, G_SETTINGS_BIND_GET)
 
   BIND_ (AUTO_INDENT,            "auto-indent");
-  BIND_ (FONT_NAME,              "font-name");
   BIND_ (SHOW_WHITESPACE,        "show-whitespace");
   BIND_ (SHOW_LINE_ENDINGS,      "show-line-endings");
   BIND_ (HIGHLIGHT_CURRENT_LINE, "highlight-current-line");
@@ -279,13 +242,17 @@ mousepad_view_init (MousepadView *view)
   BIND_ (WORD_WRAP,              "word-wrap");
   BIND_ (MATCH_BRACES,           "match-braces");
 
-  /* override with default font when the setting is enabled */
-  MOUSEPAD_SETTING_CONNECT_OBJECT (USE_DEFAULT_FONT,
-                                   G_CALLBACK (mousepad_view_use_default_font_setting_changed),
-                                   view,
-                                   G_CONNECT_SWAPPED);
-
 #undef BIND_
+
+  /* bind the "font" property conditionally */
+  mousepad_view_use_default_font (view);
+  MOUSEPAD_SETTING_CONNECT_OBJECT (USE_DEFAULT_FONT,
+                                   G_CALLBACK (mousepad_view_use_default_font),
+                                   view, G_CONNECT_SWAPPED);
+
+  /* bind the whitespace display property to that of the application */
+  application = g_application_get_default ();
+  g_object_bind_property (application, "space-location", view, "space-location", G_BINDING_SYNC_CREATE);
 }
 
 
@@ -295,20 +262,9 @@ mousepad_view_finalize (GObject *object)
 {
   MousepadView *view = MOUSEPAD_VIEW (object);
 
-  /* stop a running selection timeout */
-  if (G_UNLIKELY (view->selection_timeout_id != 0))
-    g_source_remove (view->selection_timeout_id);
-
-  /* free the selection marks list (marks are owned by the buffer) */
-  if (G_UNLIKELY (view->selection_marks != NULL))
-    g_slist_free (view->selection_marks);
-
   /* cleanup color scheme name */
   g_free (view->color_scheme);
 
-  /* cleanup font description */
-  pango_font_description_free (view->font_desc);
-
   (*G_OBJECT_CLASS (mousepad_view_parent_class)->finalize) (object);
 }
 
@@ -324,12 +280,15 @@ mousepad_view_set_property (GObject      *object,
 
   switch (prop_id)
     {
-    case PROP_FONT_NAME:
-      mousepad_view_set_font_name (view, g_value_get_string (value));
+    case PROP_FONT:
+      mousepad_view_set_font (view, g_value_get_string (value));
       break;
     case PROP_SHOW_WHITESPACE:
       mousepad_view_set_show_whitespace (view, g_value_get_boolean (value));
       break;
+    case PROP_SPACE_LOCATION:
+      mousepad_view_set_space_location_flags (view, g_value_get_flags (value));
+      break;
     case PROP_SHOW_LINE_ENDINGS:
       mousepad_view_set_show_line_endings (view, g_value_get_boolean (value));
       break;
@@ -350,45 +309,6 @@ mousepad_view_set_property (GObject      *object,
 
 
 
-static void
-mousepad_view_get_property (GObject    *object,
-                            guint       prop_id,
-                            GValue     *value,
-                            GParamSpec *pspec)
-{
-  MousepadView *view = MOUSEPAD_VIEW (object);
-  gchar        *font_name;
-
-  switch (prop_id)
-    {
-    case PROP_FONT_NAME:
-      font_name = pango_font_description_to_string (view->font_desc);
-      g_value_set_string (value, font_name);
-      g_free (font_name);
-      break;
-    case PROP_SHOW_WHITESPACE:
-      g_value_set_boolean (value, mousepad_view_get_show_whitespace (view));
-      break;
-    case PROP_SHOW_LINE_ENDINGS:
-      g_value_set_boolean (value, mousepad_view_get_show_line_endings (view));
-      break;
-    case PROP_COLOR_SCHEME:
-      g_value_set_string (value, mousepad_view_get_color_scheme (view));
-      break;
-    case PROP_WORD_WRAP:
-      g_value_set_boolean (value, mousepad_view_get_word_wrap (view));
-      break;
-    case PROP_MATCH_BRACES:
-      g_value_set_boolean (value, mousepad_view_get_match_braces (view));
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
-
-
 static gboolean
 mousepad_view_key_press_event (GtkWidget   *widget,
                                GdkEventKey *event)
@@ -398,7 +318,6 @@ mousepad_view_key_press_event (GtkWidget   *widget,
   GtkTextIter    iter;
   GtkTextMark   *cursor;
   guint          modifiers;
-  gboolean       is_editable;
 
   /* get the modifiers state */
   modifiers = event->state & gtk_accelerator_get_default_mod_mask ();
@@ -406,9 +325,6 @@ mousepad_view_key_press_event (GtkWidget   *widget,
   /* get the textview buffer */
   buffer = mousepad_view_get_buffer (view);
 
-  /* whether the textview is editable */
-  is_editable = gtk_text_view_get_editable (GTK_TEXT_VIEW (view));
-
   /* handle the key event */
   switch (event->keyval)
     {
@@ -463,17 +379,6 @@ mousepad_view_key_press_event (GtkWidget   *widget,
           }
         break;
 
-      case GDK_KEY_Delete:
-      case GDK_KEY_KP_Delete:
-        if (view->selection_marks != NULL && is_editable)
-          {
-            /* delete or destroy the selection */
-            if (view->selection_length > 0)
-              mousepad_view_delete_selection (view);
-            return TRUE;
-          }
-        break;
-
       default:
         break;
     }
@@ -899,7 +804,7 @@ mousepad_view_clipboard_cut (MousepadView *view)
   GtkTextBuffer *buffer;
 
   g_return_if_fail (MOUSEPAD_IS_VIEW (view));
-  g_return_if_fail (mousepad_view_get_selection_length (view, NULL) > 0);
+  g_return_if_fail (mousepad_view_get_selection_length (view) > 0);
 
   /* get the clipboard */
   clipboard = gtk_widget_get_clipboard (GTK_WIDGET (view), GDK_SELECTION_CLIPBOARD);
@@ -923,7 +828,7 @@ mousepad_view_clipboard_copy (MousepadView *view)
   GtkTextBuffer *buffer;
 
   g_return_if_fail (MOUSEPAD_IS_VIEW (view));
-  g_return_if_fail (mousepad_view_get_selection_length (view, NULL) > 0);
+  g_return_if_fail (mousepad_view_get_selection_length (view) > 0);
 
   /* get the clipboard */
   clipboard = gtk_widget_get_clipboard (GTK_WIDGET (view), GDK_SELECTION_CLIPBOARD);
@@ -1053,7 +958,7 @@ mousepad_view_delete_selection (MousepadView *view)
   GtkTextBuffer *buffer;
 
   g_return_if_fail (MOUSEPAD_IS_VIEW (view));
-  g_return_if_fail (mousepad_view_get_selection_length (view, NULL) > 0);
+  g_return_if_fail (mousepad_view_get_selection_length (view) > 0);
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -1091,15 +996,13 @@ void
 mousepad_view_convert_selection_case (MousepadView *view,
                                       gint          type)
 {
-  gchar         *text;
-  gchar         *converted;
   GtkTextBuffer *buffer;
   GtkTextIter    start_iter, end_iter;
-  gint           offset = -1;
-  GSList        *li = NULL;
+  gchar         *text, *converted;
+  gint           offset;
 
   g_return_if_fail (MOUSEPAD_IS_VIEW (view));
-  g_return_if_fail (mousepad_view_get_selection_length (view, NULL) > 0);
+  g_return_if_fail (mousepad_view_get_selection_length (view) > 0);
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -1107,88 +1010,59 @@ mousepad_view_convert_selection_case (MousepadView *view,
   /* begin a user action */
   gtk_text_buffer_begin_user_action (buffer);
 
-  if (view->selection_marks == NULL)
+  /* get selection bounds */
+  gtk_text_buffer_get_selection_bounds (buffer, &start_iter, &end_iter);
+
+  /* get string offset */
+  offset = gtk_text_iter_get_offset (&start_iter);
+
+  /* get the selected string */
+  text = gtk_text_buffer_get_slice (buffer, &start_iter, &end_iter, FALSE);
+  if (G_LIKELY (text != NULL))
     {
-      /* get selection bounds */
-      gtk_text_buffer_get_selection_bounds (buffer, &start_iter, &end_iter);
-
-      /* get string offset */
-      offset = gtk_text_iter_get_offset (&start_iter);
-
-      /* enter the loop without hassle */
-      goto selection_enter;
-    }
-
-  /* replace all selected items */
-  for (li = view->selection_marks; li != NULL; li = li->next)
-    {
-      /* get iters from column selection */
-      gtk_text_buffer_get_iter_at_mark (buffer, &start_iter, li->data);
-      li = li->next;
-      gtk_text_buffer_get_iter_at_mark (buffer, &end_iter, li->data);
-
-      /* label for normal selections */
-      selection_enter:
-
-      /* get the selected string */
-      text = gtk_text_buffer_get_slice (buffer, &start_iter, &end_iter, FALSE);
-      if (G_LIKELY (text != NULL))
+      switch (type)
         {
-          switch (type)
-            {
-              case LOWERCASE:
-                converted = g_utf8_strdown (text, -1);
-                break;
+          case LOWERCASE:
+            converted = g_utf8_strdown (text, -1);
+            break;
 
-              case UPPERCASE:
-                converted = g_utf8_strup (text, -1);
-                break;
+          case UPPERCASE:
+            converted = g_utf8_strup (text, -1);
+            break;
 
-              case TITLECASE:
-                converted = mousepad_util_utf8_strcapital (text);
-                break;
+          case TITLECASE:
+            converted = mousepad_util_utf8_strcapital (text);
+            break;
 
-              case OPPOSITE_CASE:
-                converted = mousepad_util_utf8_stropposite (text);
-                break;
+          case OPPOSITE_CASE:
+            converted = mousepad_util_utf8_stropposite (text);
+            break;
 
-              default:
-                g_assert_not_reached ();
-                break;
-            }
-
-          /* only update the buffer if the string changed */
-          if (G_LIKELY (converted && strcmp (text, converted) != 0))
-            {
-              /* debug check */
-              g_return_if_fail (g_utf8_validate (converted, -1, NULL));
-
-              /* delete old string */
-              gtk_text_buffer_delete (buffer, &start_iter, &end_iter);
-
-              /* insert string */
-              gtk_text_buffer_insert (buffer, &end_iter, converted, -1);
-            }
-
-          /* cleanup */
-          g_free (converted);
-          g_free (text);
+          default:
+            g_assert_not_reached ();
+            break;
         }
 
-      /* leave for normal selections */
-      if (offset != -1)
-        break;
+      /* only update the buffer if the string changed */
+      if (G_LIKELY (converted && strcmp (text, converted) != 0))
+        {
+          /* delete old string */
+          gtk_text_buffer_delete (buffer, &start_iter, &end_iter);
+
+          /* insert string */
+          gtk_text_buffer_insert (buffer, &end_iter, converted, -1);
+        }
+
+      /* cleanup */
+      g_free (converted);
+      g_free (text);
     }
 
-  /* restore selection if needed */
-  if (offset != -1)
-    {
-      /* restore start iter */
-      gtk_text_buffer_get_iter_at_offset (buffer, &start_iter, offset);
+  /* restore start iter */
+  gtk_text_buffer_get_iter_at_offset (buffer, &start_iter, offset);
 
-      /* select range */
-      gtk_text_buffer_select_range (buffer, &end_iter, &start_iter);
-    }
+  /* select range */
+  gtk_text_buffer_select_range (buffer, &end_iter, &start_iter);
 
   /* end user action */
   gtk_text_buffer_end_user_action (buffer);
@@ -1620,72 +1494,54 @@ mousepad_view_indent (MousepadView *view,
 
 
 
-gboolean
-mousepad_view_get_selection_length (MousepadView *view,
-                                    gboolean     *is_column_selection)
+gint
+mousepad_view_get_selection_length (MousepadView *view)
 {
   GtkTextBuffer *buffer;
-  GtkTextIter    sel_start, sel_end;
-  gint           sel_length = 0;
-  gboolean       column_selection;
+  GtkTextIter    start, end;
+  gint           length = 0;
 
   g_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
 
   /* get the text buffer */
   buffer = mousepad_view_get_buffer (view);
 
-  /* whether this is a column selection */
-  column_selection = view->selection_marks != NULL || view->selection_start_x != -1;
+  /* calculate the selection length from the iter offset (absolute) */
+  if (gtk_text_buffer_get_selection_bounds (buffer, &start, &end))
+    length = ABS (gtk_text_iter_get_offset (&end) - gtk_text_iter_get_offset (&start));
 
-  /* we have a vertical selection */
-  if (column_selection)
-    {
-      /* get the selection count from the selection draw function */
-      sel_length = view->selection_length;
-    }
-  else if (gtk_text_buffer_get_selection_bounds (buffer, &sel_start, &sel_end))
-    {
-      /* calculate the selection length from the iter offset (absolute) */
-      sel_length = ABS (gtk_text_iter_get_offset (&sel_end) - gtk_text_iter_get_offset (&sel_start));
-    }
-
-  /* whether this is a column selection */
-  if (is_column_selection)
-    *is_column_selection = column_selection;
-
-  /* return length */
-  return sel_length;
+  return length;
 }
 
 
 
-void
-mousepad_view_set_font_name (MousepadView *view,
-                             const gchar  *font_name)
+static void
+mousepad_view_set_font (MousepadView *view,
+                        const gchar  *font)
 {
   PangoFontDescription *font_desc;
+  GtkCssProvider       *provider;
+  gchar                *css_font_desc, *css_string;
 
   g_return_if_fail (MOUSEPAD_IS_VIEW (view));
 
-  /* Passing NULL resets to the default font */
-  if (font_name == NULL)
-    font_name = MOUSEPAD_VIEW_DEFAULT_FONT;
+  /* from font string to css string through pango description */
+  font_desc = pango_font_description_from_string (font);
+  css_font_desc = mousepad_util_pango_font_description_to_css (font_desc);
+  css_string = g_strdup_printf ("textview { %s }", css_font_desc);
 
-  font_desc = pango_font_description_from_string (font_name);
-  if (G_LIKELY (font_desc != NULL))
-    {
-      /* save the font description for later updating */
-      pango_font_description_free (view->font_desc);
-      view->font_desc = font_desc;
+  /* set font */
+  provider = gtk_css_provider_new ();
+  gtk_css_provider_load_from_data (provider, css_string, -1, NULL);
+  gtk_style_context_add_provider (gtk_widget_get_style_context (GTK_WIDGET (view)),
+                                  GTK_STYLE_PROVIDER (provider),
+                                  GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
-      /* apply either the default or this font depending on settings */
-      mousepad_view_update_font (view);
-
-      /* Notify interested parties that the property has changed */
-      g_object_notify (G_OBJECT (view), "font-name");
-    }
-  else
-    g_critical ("Invalid font-name given: %s", font_name);
+  /* cleanup */
+  g_object_unref (provider);
+  pango_font_description_free (font_desc);
+  g_free (css_font_desc);
+  g_free (css_string);
 }
 
 
@@ -1693,81 +1549,47 @@ mousepad_view_set_font_name (MousepadView *view,
 static void
 mousepad_view_update_draw_spaces (MousepadView *view)
 {
-  GtkSourceSpaceDrawer *drawer;
-  GtkSourceSpaceLocationFlags location_flags = GTK_SOURCE_SPACE_LOCATION_NONE;
-  GtkSourceSpaceTypeFlags type_flags = GTK_SOURCE_SPACE_TYPE_NONE;
-  gboolean enable_matrix = FALSE;
+  GtkSourceSpaceTypeFlags      type_flags = GTK_SOURCE_SPACE_TYPE_NONE;
+  GtkSourceSpaceLocationFlags  flag;
+  GtkSourceSpaceDrawer        *drawer;
+  gboolean                     enable_matrix = FALSE;
 
   drawer = gtk_source_view_get_space_drawer (GTK_SOURCE_VIEW (view));
 
   if (view->show_whitespace)
     {
-      location_flags = GTK_SOURCE_SPACE_LOCATION_ALL;
-      type_flags |= GTK_SOURCE_SPACE_TYPE_SPACE
-                    | GTK_SOURCE_SPACE_TYPE_TAB
+      type_flags |= GTK_SOURCE_SPACE_TYPE_SPACE | GTK_SOURCE_SPACE_TYPE_TAB
                     | GTK_SOURCE_SPACE_TYPE_NBSP;
       enable_matrix = TRUE;
+
+      for (flag = 1; flag < GTK_SOURCE_SPACE_LOCATION_ALL; flag <<= 1)
+        if (view->space_location_flags & flag)
+          gtk_source_space_drawer_set_types_for_locations (drawer, flag, type_flags);
+        else
+          gtk_source_space_drawer_set_types_for_locations (drawer, flag, GTK_SOURCE_SPACE_TYPE_NONE);
     }
+  else
+    gtk_source_space_drawer_set_types_for_locations (drawer, GTK_SOURCE_SPACE_LOCATION_ALL,
+                                                     GTK_SOURCE_SPACE_TYPE_NONE);
 
   if (view->show_line_endings)
     {
-      location_flags = GTK_SOURCE_SPACE_LOCATION_ALL;
-      type_flags |= GTK_SOURCE_SPACE_TYPE_NEWLINE;
       enable_matrix = TRUE;
+      if (view->space_location_flags & GTK_SOURCE_SPACE_LOCATION_TRAILING)
+        type_flags |= GTK_SOURCE_SPACE_TYPE_NEWLINE;
+      else
+        type_flags = GTK_SOURCE_SPACE_TYPE_NEWLINE;
+
+      gtk_source_space_drawer_set_types_for_locations (drawer, GTK_SOURCE_SPACE_LOCATION_TRAILING,
+                                                       type_flags);
     }
 
-  gtk_source_space_drawer_set_types_for_locations (drawer, location_flags, type_flags);
   gtk_source_space_drawer_set_enable_matrix (drawer, enable_matrix);
 }
 
 
 
 static void
-mousepad_view_update_font (MousepadView *view)
-{
-  PangoFontDescription *font_desc;
-  XfconfChannel        *channel;
-  GtkCssProvider       *provider;
-  GtkStyleContext      *context;
-  gchar                *font, *css_font_desc, *css_string;
-
-  context = gtk_widget_get_style_context (GTK_WIDGET (view));
-
-  /* determine which font description to use */
-  if (MOUSEPAD_SETTING_GET_BOOLEAN (USE_DEFAULT_FONT))
-    {
-      channel = xfconf_channel_get ("xsettings");
-      font = xfconf_channel_get_string (channel, "/Gtk/MonospaceFontName", NULL);
-
-      if (G_LIKELY (font != NULL))
-        {
-          font_desc = pango_font_description_from_string (font);
-          g_free (font);
-        }
-      else
-        font_desc = pango_font_description_from_string (MOUSEPAD_VIEW_DEFAULT_FONT);
-    }
-  else
-    font_desc = pango_font_description_copy (view->font_desc);
-
-  /* update font */
-  provider = gtk_css_provider_new ();
-  css_font_desc = mousepad_pango_font_description_to_css (font_desc);
-  css_string = g_strdup_printf ("textview { %s }", css_font_desc);
-  gtk_css_provider_load_from_data (provider, css_string, -1, NULL);
-  gtk_style_context_add_provider (context, GTK_STYLE_PROVIDER (provider),
-                                  GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-
-  /* cleanup */
-  pango_font_description_free (font_desc);
-  g_object_unref (provider);
-  g_free (css_font_desc);
-  g_free (css_string);
-}
-
-
-
-void
 mousepad_view_set_show_whitespace (MousepadView *view,
                                    gboolean      show)
 {
@@ -1775,22 +1597,23 @@ mousepad_view_set_show_whitespace (MousepadView *view,
 
   view->show_whitespace = show;
   mousepad_view_update_draw_spaces (view);
-  g_object_notify (G_OBJECT (view), "show-whitespace");
 }
 
 
 
-gboolean
-mousepad_view_get_show_whitespace (MousepadView *view)
+static void
+mousepad_view_set_space_location_flags (MousepadView *view,
+                                        gulong        flags)
 {
-  g_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
 
-  return view->show_whitespace;
+  view->space_location_flags = flags;
+  mousepad_view_update_draw_spaces (view);
 }
 
 
 
-void
+static void
 mousepad_view_set_show_line_endings (MousepadView *view,
                                      gboolean      show)
 {
@@ -1798,22 +1621,11 @@ mousepad_view_set_show_line_endings (MousepadView *view,
 
   view->show_line_endings = show;
   mousepad_view_update_draw_spaces (view);
-  g_object_notify (G_OBJECT (view), "show-line-endings");
 }
 
 
 
-gboolean
-mousepad_view_get_show_line_endings (MousepadView *view)
-{
-  g_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
-
-  return view->show_line_endings;
-}
-
-
-
-void
+static void
 mousepad_view_set_color_scheme (MousepadView *view,
                                 const gchar  *color_scheme)
 {
@@ -1826,24 +1638,12 @@ mousepad_view_set_color_scheme (MousepadView *view,
 
       /* update the buffer if there is one */
       mousepad_view_buffer_changed (view, NULL, NULL);
-
-      g_object_notify (G_OBJECT (view), "color-scheme");
     }
 }
 
 
 
-const gchar *
-mousepad_view_get_color_scheme (MousepadView *view)
-{
-  g_return_val_if_fail (MOUSEPAD_IS_VIEW (view), NULL);
-
-  return view->color_scheme;
-}
-
-
-
-void
+static void
 mousepad_view_set_word_wrap (MousepadView *view,
                              gboolean      enabled)
 {
@@ -1851,26 +1651,11 @@ mousepad_view_set_word_wrap (MousepadView *view,
 
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (view),
                                enabled ? GTK_WRAP_WORD_CHAR : GTK_WRAP_NONE);
-  g_object_notify (G_OBJECT (view), "word-wrap");
 }
 
 
 
-gboolean
-mousepad_view_get_word_wrap (MousepadView *view)
-{
-  GtkWrapMode mode;
-
-  g_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
-
-  mode = gtk_text_view_get_wrap_mode (GTK_TEXT_VIEW (view));
-
-  return (mode == GTK_WRAP_WORD_CHAR);
-}
-
-
-
-void
+static void
 mousepad_view_set_match_braces (MousepadView *view,
                                 gboolean      enabled)
 {
@@ -1879,16 +1664,4 @@ mousepad_view_set_match_braces (MousepadView *view,
   view->match_braces = enabled;
 
   mousepad_view_buffer_changed (view, NULL, NULL);
-
-  g_object_notify (G_OBJECT (view), "match-braces");
-}
-
-
-
-gboolean
-mousepad_view_get_match_braces (MousepadView *view)
-{
-  g_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
-
-  return view->match_braces;
 }
