@@ -37,8 +37,8 @@ enum
 
 static const GtkTargetEntry drop_targets[] =
 {
-  { (gchar *) "text/uri-list", 0, TARGET_TEXT_URI_LIST },
-  { (gchar *) "GTK_NOTEBOOK_TAB", GTK_TARGET_SAME_APP, TARGET_GTK_NOTEBOOK_TAB }
+  { "text/uri-list", 0, TARGET_TEXT_URI_LIST },
+  { "GTK_NOTEBOOK_TAB", GTK_TARGET_SAME_APP, TARGET_GTK_NOTEBOOK_TAB }
 };
 
 typedef struct _MousepadWindowClass MousepadWindowClass;
@@ -60,6 +60,8 @@ GtkWidget      *mousepad_window_get_languages_menu         (MousepadWindow
 
 void            mousepad_window_update_document_menu_items (MousepadWindow       *window);
 
+void            mousepad_window_update_window_menu_items   (MousepadWindow       *window);
+
 G_END_DECLS
 
 #endif /* !__MOUSEPAD_WINDOW_H__ */
